function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

var NodeMap =
/*#__PURE__*/
function () {
  function NodeMap() {
    _defineProperty(this, "_map", new Map());
  }

  var _proto = NodeMap.prototype;

  _proto._setMountNode = function _setMountNode(mountNode) {
    this.mountNode = mountNode;
  };

  _proto.get = function get(vnode) {
    if (!vnode) return null;
    if (vnode.nodeName === 'BODY') return this.mountNode;
    return this._map.get(vnode.$$id);
  };

  _proto.set = function set(vnode, node) {
    node.$$id = vnode.$$id;
    return this._map.set(vnode.$$id, node);
  };

  _proto.delete = function _delete(vnode) {
    if (!vnode) return null;
    return this._map.delete(vnode.$$id);
  };

  return NodeMap;
}();

// feature-detect support for event listener options
var supportsPassive = false;

try {
  addEventListener('test', null, {
    get passive() {
      supportsPassive = true;
    }

  });
} catch (e) {}

var passive = supportsPassive;

var EVENT_OPTIONS = passive ? {
  capture: true,
  passive: true
} : true;
var NO_BUBBLES_EVENTS = {
  // Resource Events and Progress Events
  load: true,
  error: true,
  unload: true,
  abort: true,
  loadstart: true,
  progress: true,
  loadend: true,
  // Focus Events
  blur: true,
  focus: true,
  // View Events
  scroll: true,
  // Not bubles on elements
  appear: true,
  disappear: true,
  // Uncategorized events
  invalid: true
};
var TOUCH_EVENTS = ['touchstart', 'touchmove', 'touchend', 'touchcancel'];

var EventHandler =
/*#__PURE__*/
function () {
  function EventHandler(handler, options) {
    if (options === void 0) {
      options = {};
    }

    this.postMessage = handler;
    this.registeredEventCounts = {};
    this.mountNode = options.mountNode || document;
    this.touch = null;
  }

  var _proto = EventHandler.prototype;

  _proto.addEvent = function addEvent(node, name) {
    if (NO_BUBBLES_EVENTS[name]) {
      this.addNoBubblesEventListener(node, name);
    } else {
      var registeredCount = this.registeredEventCounts[name];

      if (!registeredCount) {
        this.registeredEventCounts[name] = 1; // Top-level register

        this.mountNode.addEventListener(name, this.eventProxyHandler.bind(this), EVENT_OPTIONS);
      } else {
        this.registeredEventCounts[name]++;
      }
    }
  };

  _proto.removeEvent = function removeEvent(node, name) {
    if (NO_BUBBLES_EVENTS[name]) {
      this.removeNoBubblesEventListener(node, name);
    } else {
      this.registeredEventCounts[name]--;

      if (this.registeredEventCounts[name] === 0) {
        this.mountNode.removeEventListener(name, this.eventProxyHandler.bind(this));
      }
    }
  };

  _proto.addNoBubblesEventListener = function addNoBubblesEventListener(node, name) {
    var _this = this;

    function listener(evt) {
      var target = {
        $$id: node.$$id
      };

      _this.postMessage({
        type: 'event',
        event: {
          type: name,
          target: target,
          currentTarget: target,
          detail: evt.detail
        }
      });
    }
    node["__$" + name + "_listener__"] = listener;
    node.addEventListener(name, listener);
  };

  _proto.removeNoBubblesEventListener = function removeNoBubblesEventListener(node, name) {
    var listener = node["__$" + name + "_listener__"];

    if (listener) {
      node.removeEventListener(name, listener);
      node["__$" + name + "_listener__"] = null;
    }
  };

  _proto.eventProxyHandler = function eventProxyHandler(e) {
    if (e.type === 'click' && this.touch) return false;
    var event = {
      type: e.type
    };
    if (e.target) event.target = e.target.$$id;

    if (e.type === 'scroll' && e.target === document) {
      event.target = document.body.$$id; // page scroll container's top
      // safari is document.body.scrollTop
      // chrome is document.documentElement.scrollTop

      event.scrollTop = document.documentElement.scrollTop || document.body.scrollTop;
    } // CustomEvent detail


    if (e.detail) event.detail = e.detail;

    for (var i in e) {
      var v = e[i];

      if (typeof v !== 'object' && typeof v !== 'function' && i !== i.toUpperCase() && !event.hasOwnProperty(i)) {
        event[i] = v;
      }
    }

    if (TOUCH_EVENTS.indexOf(e.type) !== -1) {
      event.touches = serializeTouchList(e.touches);
      event.changedTouches = serializeTouchList(e.changedTouches);
    }

    this.postMessage({
      type: 'event',
      event: event
    });

    if (e.type === 'touchstart') {
      this.touch = getTouch(e);
    } else if (e.type === 'touchend' && this.touch) {
      var t = getTouch(e);

      if (t) {
        var delta = Math.sqrt(Math.pow(t.pageX - this.touch.pageX, 2) + Math.pow(t.pageY - this.touch.pageY, 2));

        if (delta < 10) {
          event.type = 'click';
          this.postMessage({
            type: 'event',
            event: event
          });
        }
      }
    }
  };

  return EventHandler;
}();

function getTouch(e) {
  var t = e.changedTouches && e.changedTouches[0] || e.touches && e.touches[0] || e;
  return t && {
    pageX: t.pageX,
    pageY: t.pageY
  };
}

function serializeTouchList(touchList) {
  var touches = [];

  for (var i = 0, l = touchList.length; i < l; i++) {
    var _touchList$i = touchList[i],
        clientX = _touchList$i.clientX,
        clientY = _touchList$i.clientY,
        pageX = _touchList$i.pageX,
        pageY = _touchList$i.pageY,
        identifier = _touchList$i.identifier,
        target = _touchList$i.target;
    touches.push({
      clientX: clientX,
      clientY: clientY,
      pageX: pageX,
      pageY: pageY,
      identifier: identifier,
      // instance id of changed target
      $$id: target.$$id
    });
  }

  return touches;
}

var CUSTOM_TAG_PREFIX = 'a-';
/**
 * Whitelist that declear w3c standard tag names.
 */

var w3cTags = {
  style: true
};

function isValidW3CTag(tagName) {
  return w3cTags.hasOwnProperty(tagName);
}
/**
 * Format tag name.
 */


function getTagName(nodeName) {
  var tagName = String(nodeName).toLowerCase();

  if (isValidW3CTag(tagName)) {
    return tagName;
  } else {
    return CUSTOM_TAG_PREFIX + tagName;
  }
}
function createNode(vnode) {
  var node, tagName;

  if (vnode.nodeType === 3) {
    node = document.createTextNode(vnode.data);
  } else if (vnode.nodeType === 1 && (tagName = getTagName(vnode.nodeName))) {
    node = document.createElement(tagName);
  } else if (vnode.nodeType === 8) {
    node = document.createComment(vnode.data);
  }

  return node;
}

var PREFIX_PROPS = {
  flex: true,
  alignItems: true,
  alignSelf: true,
  flexDirection: true,
  justifyContent: true,
  flexWrap: true,
  lineClamp: true,
  textSizeAdjust: true,
  textDecorationLine: true,
  textDecorationColor: true,
  textDecorationStyle: true,
  textDecorationSkip: true,
  writingMode: true,
  animatin: true,
  animationName: true,
  animationDuration: true,
  animationTimingFunction: true,
  animationDelay: true,
  animationIterationCount: true,
  animationDirection: true,
  animationFillMode: true,
  animationPlayState: true,
  transform: true,
  transformOrigin: true,
  transformStyle: true,
  perspective: true,
  perspectiveOrigin: true,
  backfaceVisibility: true,
  appearance: true,
  userSelect: true,
  columns: true,
  columnWidth: true,
  columnCount: true,
  columnGap: true,
  columnRule: true,
  columnRuleWidth: true,
  columnRuleStyle: true,
  columnRuleColor: true,
  columnSpan: true,
  columnFill: true,
  columnBreakBefore: true,
  columnBreakAfter: true,
  columnBreakInside: true
};
var PREFIX_PROP_VALS = {
  position: 'sticky',
  display: 'flex'
};
var StylePrefixer = {
  shouldPrefix: function shouldPrefix(prop) {
    return PREFIX_PROPS[prop] || PREFIX_PROP_VALS[prop];
  }
};
Object.keys(PREFIX_PROPS).forEach(function (prop) {
  var vendorProp = 'webkit' + prop[0].toUpperCase() + prop.slice(1);

  StylePrefixer[prop] = function (value, style) {
    if (style === void 0) {
      style = {};
    }

    style[vendorProp] = value;
    style[prop] = value;
    return style;
  };
});
Object.keys(PREFIX_PROP_VALS).forEach(function (prop) {
  var rule = PREFIX_PROP_VALS[prop];

  StylePrefixer[prop] = function (value, style) {
    if (style === void 0) {
      style = {};
    }

    if (value === rule) {
      style[prop] = ['-webkit-' + rule, rule];
    } else {
      style[prop] = value;
    }

    return style;
  };
});
function setStyle(node, styleObject) {
  var tranformedStyles = {};

  for (var prop in styleObject) {
    var val = styleObject[prop];

    if (StylePrefixer.shouldPrefix(prop)) {
      StylePrefixer[prop](val, tranformedStyles);
    } else {
      tranformedStyles[prop] = val;
    }
  }

  for (var _prop in tranformedStyles) {
    var transformValue = tranformedStyles[_prop]; // if browser only accept -webkit-flex
    // node.style.display = 'flex' will not work

    if (Array.isArray(transformValue)) {
      for (var i = 0; i < transformValue.length; i++) {
        node.style[_prop] = transformValue[i];
      }
    } else {
      node.style[_prop] = transformValue;
    }
  }
}

var CAMELCASE_REG = /-[a-z]/g;
var CamelCaseCache = {};
function camelCase(str) {
  return CamelCaseCache[str] || (CamelCaseCache[str] = str.replace(CAMELCASE_REG, function ($1) {
    return $1.slice(1).toUpperCase();
  }));
}

var KebabCache = {};
function kebabCase(string) {
  if (KebabCache[string]) {
    return KebabCache[string];
  }

  var kebab = '';

  for (var i = 0, l = string.length; i < l; i++) {
    if (/[a-z]/.test(string[i])) {
      kebab += string[i];
    } else if (/[A-Z]/.test(string[i])) {
      kebab += '-' + string[i].toLowerCase();
    } else if (string[i] === '-') {
      kebab += '-';
    }
  }

  return KebabCache[string] = kebab;
}

var STYLE = 'style';
var INVALID_ATTRS_MAP = {
  /**
   * In web components, slot attribute has side effects to
   * deliver child nodes.
   */
  slot: true
};
function isInvalidAttr(attr) {
  return INVALID_ATTRS_MAP.hasOwnProperty(attr);
}
function setAttribute(node, attrName, value) {
  if (isInvalidAttr(attrName)) return;

  if (attrName === STYLE) {
    setStyle(node, value);
    return;
  }

  if (typeof value === 'object' || typeof value === 'boolean') {
    /**
     * Transform kebab cased attribute name to camel cased property name.
     * <a-view foo-bar="true" />
     * ->
     * el.fooBar = true;
     */
    node[camelCase(attrName)] = value;
  } else {
    attrName = kebabCase(attrName);

    if (value == null) {
      node.removeAttribute(attrName);
    } else {
      node.setAttribute(attrName, value);
    }
  }
}

var STYLE_ELEMENT = 'STYLE';
var TEXT_CONTENT = 'textContent';
var TEXT_CONTENT_ATTR = TEXT_CONTENT in document ? TEXT_CONTENT : 'nodeValue';
/**
 * Returns "attributes" if it was an attribute mutation.
 * "characterData" if it was a mutation to a CharacterData node.
 * And "childList" if it was a mutation to the tree of nodes.
 */

var MutationHandler =
/*#__PURE__*/
function () {
  function MutationHandler(sender, mountNode) {
    this.eventHandler = new EventHandler(sender, {
      mountNode: mountNode
    });
    this.mountNode = mountNode || document.body;
    this.sharedNodeMap = new NodeMap();

    this.sharedNodeMap._setMountNode(this.mountNode);
  }

  var _proto = MutationHandler.prototype;

  _proto.apply = function apply(data) {
    var mutations = data.mutations;

    for (var i = 0; i < mutations.length; i++) {
      // apply mutation
      var mutation = mutations[i];
      this[mutation.type](mutation);
    }
  };

  _proto.childList = function childList(_ref) {
    var target = _ref.target,
        removedNodes = _ref.removedNodes,
        addedNodes = _ref.addedNodes,
        nextSibling = _ref.nextSibling;
    var sharedNodeMap = this.sharedNodeMap;
    var vnode = target;

    if (vnode && vnode.nodeName === 'BODY') {
      this.mountNode.$$id = vnode.$$id;
    }

    var parent = sharedNodeMap.get(vnode);

    if (removedNodes) {
      for (var i = removedNodes.length; i--;) {
        var node = sharedNodeMap.get(removedNodes[i]);
        /**
         * @NOTE For performance:
         *   do not remove style element or textContent of style element.
         */

        if (parent && parent.nodeName !== STYLE_ELEMENT && node && node.nodeName !== STYLE_ELEMENT) {
          sharedNodeMap.delete(node);
          parent.removeChild(node);
        }
      }
    }

    if (addedNodes) {
      for (var _i = 0; _i < addedNodes.length; _i++) {
        var newNode = sharedNodeMap.get(addedNodes[_i]);

        if (!newNode) {
          newNode = this.createNode(addedNodes[_i]);
        }
        /**
         * @NOTE Append style element to document.body, to prevent from
         *   being removed by it's parent was removed.
         */


        if (newNode.nodeName === STYLE_ELEMENT) {
          parent = document.body;
          nextSibling = null;
        }

        if (parent) {
          var siblingElement = nextSibling && sharedNodeMap.get(nextSibling) || null;

          if (siblingElement && siblingElement.nodeName === STYLE_ELEMENT) {
            siblingElement = null;
          }

          parent.insertBefore(newNode, siblingElement);
        }
      }
    }
  };

  _proto.createNode = function createNode$$1(vnode) {
    var node = createNode(vnode);

    if (vnode.nodeType === 1 && getTagName(vnode.nodeName)) {
      if (vnode.className) {
        node.className = vnode.className;
      }

      if (vnode.style) {
        setStyle(node, vnode.style);
      }

      if (vnode.attributes) {
        for (var i = 0; i < vnode.attributes.length; i++) {
          var _vnode$attributes$i = vnode.attributes[i],
              name = _vnode$attributes$i.name,
              value = _vnode$attributes$i.value;
          setAttribute(node, name, value);
        }
      }

      if (vnode.childNodes) {
        for (var _i2 = 0; _i2 < vnode.childNodes.length; _i2++) {
          node.appendChild(this.createNode(vnode.childNodes[_i2]));
        }
      }

      if (vnode.events) {
        for (var _i3 = 0; _i3 < vnode.events.length; _i3++) {
          this.eventHandler.addEvent(node, vnode.events[_i3]);
        }
      }
    }

    this.sharedNodeMap.set(vnode, node);
    return node;
  };

  _proto.attributes = function attributes(_ref2) {
    var target = _ref2.target,
        attributeName = _ref2.attributeName,
        newValue = _ref2.newValue,
        style = _ref2.style;
    var node = this.sharedNodeMap.get(target); // Node maybe null when node is removed and there is a setInterval change the node that will cause error

    if (!node) return;

    if (style) {
      setAttribute(node, 'style', style);
    }

    if (attributeName) {
      setAttribute(node, attributeName, newValue);
    }
  };

  _proto.characterData = function characterData(_ref3) {
    var target = _ref3.target,
        newValue = _ref3.newValue;
    var node = this.sharedNodeMap.get(target);
    node[TEXT_CONTENT_ATTR] = newValue;
  };

  _proto.addEvent = function addEvent(_ref4) {
    var target = _ref4.target,
        eventName = _ref4.eventName;
    var node = this.sharedNodeMap.get(target);
    if (!node) return;
    this.eventHandler.addEvent(node, eventName);
  };

  _proto.removeEvent = function removeEvent(_ref5) {
    var target = _ref5.target,
        eventName = _ref5.eventName;
    var node = this.sharedNodeMap.get(target);
    if (!node) return;
    this.eventHandler.removeEvent(node, eventName);
  };

  _proto.canvasRenderingContext2D = function canvasRenderingContext2D(_ref6) {
    var target = _ref6.target,
        method = _ref6.method,
        args = _ref6.args,
        properties = _ref6.properties;
    var canvas = this.sharedNodeMap.get(target);
    if (!canvas) return;
    var context = canvas.getContext('2d');

    if (properties) {
      for (var key in properties) {
        if (properties.hasOwnProperty(key)) {
          context[key] = properties[key];
        }
      }
    }

    if (method) {
      context[method].apply(context, args);
    }
  };

  return MutationHandler;
}();

var EvaluationHandler =
/*#__PURE__*/
function () {
  function EvaluationHandler(sender) {
    this.sender = sender;
  }

  var _proto = EvaluationHandler.prototype;

  _proto.apply = function apply(_ref) {
    var data = _ref.data;
    var type = data.type;
    this[type](data);
  };

  _proto.returnSuccess = function returnSuccess(id, success) {
    if (typeof success === 'object') {
      try {
        success = JSON.parse(JSON.stringify(success));
      } catch (e) {
        success = success.toString();
      }
    }

    if (typeof success === 'function') {
      success = success.toString();
    }

    this.send({
      type: 'success',
      id: id,
      success: success
    });
  };

  _proto.returnError = function returnError(id, _ref2) {
    var name = _ref2.name,
        message = _ref2.message;
    this.send({
      type: 'error',
      id: id,
      error: {
        name: name,
        message: message
      }
    });
  };

  _proto.send = function send(data) {
    this.sender({
      type: 'return',
      return: data
    });
  };

  _proto.eval = function _eval(_ref3) {
    var id = _ref3.id,
        code = _ref3.code;

    try {
      this.returnSuccess(id, window.eval(code));
    } catch (error) {
      this.returnError(id, error);
    }
  };

  return EvaluationHandler;
}();

/**
 * Deperated location handler.
 */
var LocationHandler =
/*#__PURE__*/
function () {
  function LocationHandler(sender) {
    this.sender = sender;
  }

  var _proto = LocationHandler.prototype;

  _proto.apply = function apply(data) {
    var payload = data.data;
    var type = payload.type,
        prop = payload.prop;

    if (type === 'call' && prop === 'replace') {
      location.replace(payload.args[0]);
    }
  };

  return LocationHandler;
}();

/* global DEVICE_WIDTH */
var index = (function (_ref) {
  var worker = _ref.worker,
      mountNode = _ref.mountNode;
  var postMessage = worker.postMessage.bind(worker);
  var handlers = {
    MutationRecord: new MutationHandler(postMessage, mountNode),
    EvaluationRecord: new EvaluationHandler(postMessage),
    // Deprecated handler.
    Location: new LocationHandler(postMessage)
  };

  worker.onmessage = function (_ref2) {
    var data = _ref2.data;
    var type = data.type;

    if (handlers[type]) {
      handlers[type].apply(data);
    } else {
      console.error('Can not handle with ' + type, data);
    }
  };

  worker.postMessage({
    type: 'init',
    url: location.href,
    width: getDeviceWidth()
  });
});
/**
 * Get device base width
 * @return {number}
 */

function getDeviceWidth() {
  return typeof DEVICE_WIDTH !== 'undefined' ? DEVICE_WIDTH : document.documentElement.clientWidth;
}

export default index;
//# sourceMappingURL=driver-worker-renderer-webview.mjs.map
